use anyhow::{Error, Result};
use config::Config;
use serde::Deserialize;

#[derive(Deserialize, Debug, Default)]
pub struct Settings {
    pub title: String,
    pub base_url: String,
    pub bind_address: String,
    pub logging: String,
    pub otlp: Otlp,
}

#[derive(Deserialize, Debug, Default)]
pub struct Otlp {
    pub enabled: bool,
    pub endpoint: String,
    pub authorization: String,
    pub organization: String,
    pub stream_name: String,
    pub tls_insecure: bool,
}

pub fn get() -> Result<Settings, Error> {
    let settings = Config::builder()
        .add_source(config::File::with_name("config.toml"))
        .add_source(config::Environment::with_prefix("TLX"))
        .build()?
        .try_deserialize()?;
    Ok(settings)
}
